%% T-61.2010 Datasta tietoon / T#2 / 1
% Jukka Parviainen, 14.11.2006

%% Pkomponenttianalyysi / Principal Component Analysis
% Otetaan perusesimerkki paperilaskarikierroksen #2 
% tehtvst 1

%% Data
% Paperilaskaritehtvn 4 pistett 2D-tasossa.

X = [2 5 6 7; ...
    1 3 5 7];
disp('Alkuperinen datamatriisi X:');
disp(X);

%% Piirretn (x1, x2)-koordinaatistoon

figure(11); clf;
p1 = plot(X(1,:), X(2,:), 'bo');
set(p1, 'MarkerFaceColor',[0 0 1]);
grid on;
axis equal
set(gca,'XTick',[-2:10]);
set(gca,'YTick',[-2:10]);
axis([-1 9 -1 9]);
title('Datapisteet');

for k = [1 : size(X, 2)]
    text(X(1,k),X(2,k),['(' num2str(X(1,k)) ',' num2str(X(2,k)) ')']);
end;

%% Datan keskiarvoistaminen

ka = (mean(X,2));
XK = X - repmat(ka, 1, size(X,2));

disp('Keskiarvo ka:');
disp(ka);
disp('joka poistetaan ja saadaan XK:');
disp(XK);

figure(11); hold on;
XC = ka(1)+[-3 0; 3 0];
YC = ka(2)+[0 -3; 0 3];
lin8 = line(XC, YC);
set(lin8,'Color',[0 1 0]);
title(['Keskiarvoistetut datapisteet, ka = (' num2str(ka(1)) ',' num2str(ka(2)) ')']);

for k = [1 : size(X, 2)]
    t1 = text(X(1,k),X(2,k)-0.5,['(' num2str(XK(1,k)) ',' num2str(XK(2,k)) ')']);
    set(t1,'Color',[0 1 0]);
end;

%% Etsi pkomponentit
% Tehtvn etsi ensin siis se suunta, jossa
% datan variaatio on suurin


%% Muodosteaan kovarianssimatriisi
% Kovarianssimatriisin kooksi tulee (d x d) eli tss 
% tapauksessa (2 x 2).
% Kerroin 1/(n-1) tuottaa harhattoman kovarianssimatriisin.

C = (1/4)*XK*XK';
disp('Kovarianssimatriisi C:');
disp(C);


%% Lasketaan ominaisarvot
% Ominaisarvot ja -vektorit lasketaan nelimatriisista.
% Ratkaistaan siis yhtl:
%$$A x = \lambda x$$
%
% Matlabissa: lue "help eig"

[V, D] = eig(C);

disp('C:n ominaisvektorit sarakkeissa:');
disp(V);

disp('C:n ominaisarvot diagonaalilla:');
disp(D);

%% Jrjestetn ominaisvektorit ominaisarvojen suuruusjrjestykseen
% Tmn jlkeen ollaan varmoja, ett V:n ensimmisess sarakkeessa
% on ominaisvektori, jota vastaava ominaisarvo on suurin.
% Mys muut vektorit ovat ominaisarvojen suuruusjrjestyksess.

[dummy1, ind] = sort(diag(D));
ind = flipud(ind);
V = V(:, ind);
D = D(:, ind);


%% Ominaisarvot
% Lasketaan kumulatiivinen summa ominaisarvoista.
%
% Tllin ensimmiset m ominaisarvoa selitt f(m) prosenttia
% koko datan vaihtelusta. Tt voidaan kytt dimension
% pudotukseen, koska useimmiten "viimeisten" dimensioiden
% tuottama varianssi on varsin pient.

lambda = sum(D);
lambdacs = cumsum(lambda);
lambdasum = sum(lambda);

disp('Ominaisarvo #k | Kumul. summa ominaisarvoista %');
disp([ [1:length(lambda)]'  100*lambdacs'/lambdasum]);

%% Projisoidaan 2D-data 1D-avaruuteen
% Kytetn projektiomatriisina suurinta pkomponenttia.

disp('PCA1-vektori:');
disp(V(:,1))
disp('Datan varianssista selittyy (%):');
disp(100*sum(D(:,1))/sum(D(:)));

%P  = V(:,1)' * X
PK = V(:,1)' * XK;
disp('Projektiopisteiden arvot 1D:ss:');
disp(PK);

%% Piirretn projektiopisteet viel 2D-avaruuteen
% Tulevat suoralle, jonka suurin ominaisvektori mr.
%
% Jos ajatellaan dekoodaamisena, niin meill on
% 4 kpl 1D-pisteit (4 yksikk) ja tarvittavat
% ominaisvektorit (2 yksikk) sen sijaan ett 
% meill olisi alkuperiset 4 kpl 2D-pisteit (8 yksikk).

XP = PK * V(1,1) + ka(1);
YP = PK * V(2,1) + ka(2);
p2 = plot(XP, YP, 'ro', 'MarkerSize', 12);

for k = [1 : size(XP, 2)]
    t1 = text(XP(k),YP(k),['(' num2str(XP(k)) ',' num2str(YP(k)) ')']);
    set(t1,'Color',[1 0 0]);
end;

p3 = line([-V(1,1) V(1,1)]*10+ka(1), [-V(2,1) V(2,1)]*10+ka(2));
set(p3, 'LineStyle', '-.', 'Color', [1 0 0]);

disp('Projektiopisteiden arvot 2D:ss:');
disp([XP; YP]);



